@@ -28,16 +28,19 @@ import (
 	"k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/intstr"
+	"knative.dev/pkg/controller"
+	"knative.dev/pkg/logging"
+	"knative.dev/pkg/system"
+
 	kafkav1beta1 "knative.dev/eventing-kafka/pkg/apis/messaging/v1beta1"
 	commonenv "knative.dev/eventing-kafka/pkg/channel/distributed/common/env"
 	"knative.dev/eventing-kafka/pkg/channel/distributed/common/health"
 	"knative.dev/eventing-kafka/pkg/channel/distributed/controller/constants"
 	"knative.dev/eventing-kafka/pkg/channel/distributed/controller/event"
 	"knative.dev/eventing-kafka/pkg/channel/distributed/controller/util"
-	"knative.dev/pkg/controller"
-	"knative.dev/pkg/logging"
-	"knative.dev/pkg/system"
+	commonconstants "knative.dev/eventing-kafka/pkg/common/constants"
 )
 
 //
@@ -121,15 +124,17 @@ func (r *Reconciler) finalizeDispatcher(ctx context.Context, channel *kafkav1bet
 // Reconcile The Dispatcher Service
 func (r *Reconciler) reconcileDispatcherService(ctx context.Context, logger *zap.Logger, channel *kafkav1beta1.KafkaChannel) error {
 
+	// Create A New Service For Comparison
+	newService := r.newDispatcherService(channel)
+
 	// Attempt To Get The Dispatcher Service Associated With The Specified Channel
-	service, err := r.getDispatcherService(channel)
-	if service == nil || err != nil {
+	existingService, err := r.getDispatcherService(channel)
+	if existingService == nil || err != nil {
 
 		// If The Service Was Not Found - Then Create A New One For The Channel
 		if errors.IsNotFound(err) {
 			logger.Info("Dispatcher Service Not Found - Creating New One")
-			service := r.newDispatcherService(channel)
-			_, err = r.kubeClientset.CoreV1().Services(service.Namespace).Create(ctx, service, metav1.CreateOptions{})
+			_, err = r.kubeClientset.CoreV1().Services(newService.Namespace).Create(ctx, newService, metav1.CreateOptions{})
 			if err != nil {
 				logger.Error("Failed To Create Dispatcher Service", zap.Error(err))
 				return err
@@ -144,19 +149,35 @@ func (r *Reconciler) reconcileDispatcherService(ctx context.Context, logger *zap
 	} else {
 
 		// Log Deletion Timestamp & Finalizer State
-		if service.DeletionTimestamp.IsZero() {
+		if existingService.DeletionTimestamp.IsZero() {
 			logger.Info("Successfully Verified Dispatcher Service")
 		} else {
-			if util.HasFinalizer(r.finalizerName(), &service.ObjectMeta) {
+			if util.HasFinalizer(r.finalizerName(), &newService.ObjectMeta) {
 				logger.Info("Blocking Pending Deletion Of Dispatcher Service (Finalizer Detected)")
 			} else {
 				logger.Warn("Unable To Block Pending Deletion Of Dispatcher Service (Finalizer Missing)")
 			}
 		}
 
-		// Return Success
-		return nil
+		// Determine whether the existing service is different in a way that demands a patch
+		// such as missing required labels or spec differences
+		patch, needsUpdate := util.CheckServiceChanged(logger, existingService, newService)
+
+		// Patch the service in Kubernetes if necessary
+		if needsUpdate {
+			_, err = r.kubeClientset.CoreV1().Services(existingService.Namespace).Patch(ctx, existingService.Name, types.JSONPatchType, patch, metav1.PatchOptions{})
+			if err == nil {
+				controller.GetEventRecorder(ctx).Event(channel, corev1.EventTypeNormal, event.DispatcherServicePatched.String(), "Dispatcher Service Patched")
+				logger.Info("Dispatcher Service Changed - Patch Applied")
+			} else {
+				controller.GetEventRecorder(ctx).Event(channel, corev1.EventTypeWarning, event.DispatcherServicePatchFailed.String(), "Dispatcher Service Patch Failed")
+				channel.Status.MarkDispatcherFailed(event.DispatcherServicePatchFailed.String(), "Failed To Patch Dispatcher Service: %v", err)
+				logger.Error("Dispatcher Service Patch Failed", zap.Error(err))
+				return err
+			}
+		}
 	}
+	return nil
 }
 
 // Finalize The Dispatcher Service
@@ -264,31 +285,32 @@ func (r *Reconciler) newDispatcherService(channel *kafkav1beta1.KafkaChannel) *c
 // Reconcile The Dispatcher Deployment
 func (r *Reconciler) reconcileDispatcherDeployment(ctx context.Context, logger *zap.Logger, channel *kafkav1beta1.KafkaChannel) error {
 
+	// Create A New Deployment For Comparison
+	newDeployment, err := r.newDispatcherDeployment(logger, channel)
+	if err != nil {
+		logger.Error("Failed To Create Dispatcher Deployment YAML", zap.Error(err))
+		channel.Status.MarkDispatcherFailed(event.DispatcherDeploymentReconciliationFailed.String(), "Failed To Generate Dispatcher Deployment: %v", err)
+		return err
+	}
+
 	// Attempt To Get The Dispatcher Deployment Associated With The Specified Channel
-	deployment, err := r.getDispatcherDeployment(channel)
-	if deployment == nil || err != nil {
+	existingDeployment, err := r.getDispatcherDeployment(channel)
+	if existingDeployment == nil || err != nil {
 
 		// If The Dispatcher Deployment Was Not Found - Then Create A New Deployment For The Channel
 		if errors.IsNotFound(err) {
 
 			// Then Create The New Deployment
 			logger.Info("Dispatcher Deployment Not Found - Creating New One")
-			deployment, err = r.newDispatcherDeployment(logger, channel)
+			newDeployment, err = r.kubeClientset.AppsV1().Deployments(newDeployment.Namespace).Create(ctx, newDeployment, metav1.CreateOptions{})
 			if err != nil {
-				logger.Error("Failed To Create Dispatcher Deployment YAML", zap.Error(err))
-				channel.Status.MarkDispatcherFailed(event.DispatcherDeploymentReconciliationFailed.String(), "Failed To Generate Dispatcher Deployment: %v", err)
+				logger.Error("Failed To Create Dispatcher Deployment", zap.Error(err))
+				channel.Status.MarkDispatcherFailed(event.DispatcherDeploymentReconciliationFailed.String(), "Failed To Create Dispatcher Deployment: %v", err)
 				return err
 			} else {
-				deployment, err = r.kubeClientset.AppsV1().Deployments(deployment.Namespace).Create(ctx, deployment, metav1.CreateOptions{})
-				if err != nil {
-					logger.Error("Failed To Create Dispatcher Deployment", zap.Error(err))
-					channel.Status.MarkDispatcherFailed(event.DispatcherDeploymentReconciliationFailed.String(), "Failed To Create Dispatcher Deployment: %v", err)
-					return err
-				} else {
-					logger.Info("Successfully Created Dispatcher Deployment")
-					channel.Status.PropagateDispatcherStatus(&deployment.Status)
-					return nil
-				}
+				logger.Info("Successfully Created Dispatcher Deployment")
+				channel.Status.PropagateDispatcherStatus(&newDeployment.Status)
+				return nil
 			}
 		} else {
 			// Failed In Attempt To Get Deployment From K8S
@@ -299,18 +321,36 @@ func (r *Reconciler) reconcileDispatcherDeployment(ctx context.Context, logger *
 	} else {
 
 		// Log Deletion Timestamp & Finalizer State
-		if deployment.DeletionTimestamp.IsZero() {
+		if existingDeployment.DeletionTimestamp.IsZero() {
 			logger.Info("Successfully Verified Dispatcher Deployment")
 		} else {
-			if util.HasFinalizer(r.finalizerName(), &deployment.ObjectMeta) {
+			if util.HasFinalizer(r.finalizerName(), &newDeployment.ObjectMeta) {
 				logger.Info("Blocking Pending Deletion Of Dispatcher Deployment (Finalizer Detected)")
 			} else {
 				logger.Warn("Unable To Block Pending Deletion Of Dispatcher Deployment (Finalizer Missing)")
 			}
 		}
 
+		// Determine whether the existing deployment is different in a way that demands an update
+		// such as missing required labels, a different image, or certain container differences.
+		// (This includes the configmap hash annotation, so configmap changes will trigger updates)
+		updatedDeployment, needsUpdate := util.CheckDeploymentChanged(logger, existingDeployment, newDeployment)
+
+		// Update the deployment in Kubernetes if necessary
+		if needsUpdate {
+			updatedDeployment, err = r.kubeClientset.AppsV1().Deployments(newDeployment.Namespace).Update(ctx, updatedDeployment, metav1.UpdateOptions{})
+			if err == nil {
+				controller.GetEventRecorder(ctx).Event(channel, corev1.EventTypeNormal, event.DispatcherDeploymentUpdated.String(), "Dispatcher Deployment Updated")
+				logger.Info("Dispatcher Deployment Changed - Update Applied")
+			} else {
+				controller.GetEventRecorder(ctx).Event(channel, corev1.EventTypeWarning, event.DispatcherDeploymentUpdateFailed.String(), "Dispatcher Deployment Update Failed")
+				channel.Status.MarkDispatcherFailed(event.DispatcherDeploymentUpdateFailed.String(), "Failed To Update Dispatcher Deployment: %v", err)
+				logger.Info("Dispatcher Deployment Failed", zap.Error(err))
+				return err
+			}
+		}
 		// Propagate Status & Return Success
-		channel.Status.PropagateDispatcherStatus(&deployment.Status)
+		channel.Status.PropagateDispatcherStatus(&updatedDeployment.Status)
 		return nil
 	}
 }
@@ -377,7 +417,7 @@ func (r *Reconciler) newDispatcherDeployment(logger *zap.Logger, channel *kafkav
 	deploymentName := util.DispatcherDnsSafeName(channel)
 
 	// Replicas Int Value For De-Referencing
-	replicas := int32(r.config.Dispatcher.Replicas)
+	replicas := int32(r.config.Channel.Dispatcher.Replicas)
 
 	// Create The Dispatcher Container Environment Variables
 	envVars, err := r.dispatcherDeploymentEnvVars(channel)
@@ -391,18 +431,28 @@ func (r *Reconciler) newDispatcherDeployment(logger *zap.Logger, channel *kafkav
 	// If we want "no limit" or "no request" then the entry must not be present in the map.
 	// Note: Since a "Quantity" type has no nil value, we use the Zero value to represent unlimited.
 	resourceLimits := make(map[corev1.ResourceName]resource.Quantity)
-	if !r.config.Dispatcher.MemoryLimit.IsZero() {
-		resourceLimits[corev1.ResourceMemory] = r.config.Dispatcher.MemoryLimit
+	if !r.config.Channel.Dispatcher.MemoryLimit.IsZero() {
+		resourceLimits[corev1.ResourceMemory] = r.config.Channel.Dispatcher.MemoryLimit
 	}
-	if !r.config.Dispatcher.CpuLimit.IsZero() {
-		resourceLimits[corev1.ResourceCPU] = r.config.Dispatcher.CpuLimit
+	if !r.config.Channel.Dispatcher.CpuLimit.IsZero() {
+		resourceLimits[corev1.ResourceCPU] = r.config.Channel.Dispatcher.CpuLimit
 	}
 	resourceRequests := make(map[corev1.ResourceName]resource.Quantity)
-	if !r.config.Dispatcher.MemoryRequest.IsZero() {
-		resourceRequests[corev1.ResourceMemory] = r.config.Dispatcher.MemoryRequest
+	if !r.config.Channel.Dispatcher.MemoryRequest.IsZero() {
+		resourceRequests[corev1.ResourceMemory] = r.config.Channel.Dispatcher.MemoryRequest
 	}
-	if !r.config.Dispatcher.CpuRequest.IsZero() {
-		resourceRequests[corev1.ResourceCPU] = r.config.Dispatcher.CpuRequest
+	if !r.config.Channel.Dispatcher.CpuRequest.IsZero() {
+		resourceRequests[corev1.ResourceCPU] = r.config.Channel.Dispatcher.CpuRequest
+	}
+
+	// If either the limits or requests are an entirely-empty map, this will be translated to a nil
+	// value in the resource itself, so pre-emptively set that here.  This prevents the CheckDeploymentChanged
+	// function from returning a difference and restarting the dispatcher unnecessarily.
+	if len(resourceLimits) == 0 {
+		resourceLimits = nil
+	}
+	if len(resourceRequests) == 0 {
+		resourceRequests = nil
 	}
 
 	// Create The Dispatcher's Deployment
@@ -434,7 +484,13 @@ func (r *Reconciler) newDispatcherDeployment(logger *zap.Logger, channel *kafkav
 			Template: corev1.PodTemplateSpec{
 				ObjectMeta: metav1.ObjectMeta{
 					Labels: map[string]string{
-						constants.AppLabel: deploymentName, // Matched By Deployment Selector Above
+						constants.AppLabel:                    deploymentName,    // Matched By Deployment Selector Above
+						constants.KafkaChannelDispatcherLabel: "true",            // Identifies the Pod as being a KafkaChannel "Dispatcher"
+						constants.KafkaChannelNameLabel:       channel.Name,      // Identifies the Pod's Owning KafkaChannel's Name
+						constants.KafkaChannelNamespaceLabel:  channel.Namespace, // Identifies the Pod's Owning KafkaChannel's Namespace
+					},
+					Annotations: map[string]string{
+						commonconstants.ConfigMapHashAnnotationKey: r.kafkaConfigMapHash,
 					},
 				},
 				Spec: corev1.PodSpec{
@@ -451,6 +507,9 @@ func (r *Reconciler) newDispatcherDeployment(logger *zap.Logger, channel *kafkav
 								},
 								InitialDelaySeconds: constants.DispatcherLivenessDelay,
 								PeriodSeconds:       constants.DispatcherLivenessPeriod,
+								TimeoutSeconds:      constants.DispatcherLivenessTimeout,
+								SuccessThreshold:    constants.DispatcherLivenessSuccessThreshold,
+								FailureThreshold:    constants.DispatcherLivenessFailureThreshold,
 							},
 							ReadinessProbe: &corev1.Probe{
 								Handler: corev1.Handler{
@@ -461,6 +520,9 @@ func (r *Reconciler) newDispatcherDeployment(logger *zap.Logger, channel *kafkav
 								},
 								InitialDelaySeconds: constants.DispatcherReadinessDelay,
 								PeriodSeconds:       constants.DispatcherReadinessPeriod,
+								TimeoutSeconds:      constants.DispatcherReadinessTimeout,
+								SuccessThreshold:    constants.DispatcherReadinessSuccessThreshold,
+								FailureThreshold:    constants.DispatcherReadinessFailureThreshold,
 							},
 							Image:           r.environment.DispatcherImage,
 							Env:             envVars,
@@ -469,6 +531,22 @@ func (r *Reconciler) newDispatcherDeployment(logger *zap.Logger, channel *kafkav
 								Limits:   resourceLimits,
 								Requests: resourceRequests,
 							},
+							VolumeMounts: []corev1.VolumeMount{
+								{
+									Name:      commonconstants.SettingsConfigMapName,
+									MountPath: commonconstants.SettingsConfigMapMountPath,
+								},
+							},
+						},
+					},
+					Volumes: []corev1.Volume{
+						{
+							Name: commonconstants.SettingsConfigMapName,
+							VolumeSource: corev1.VolumeSource{ConfigMap: &corev1.ConfigMapVolumeSource{
+								LocalObjectReference: corev1.LocalObjectReference{
+									Name: commonconstants.SettingsConfigMapName,
+								},
+							}},
 						},
 					},
 				},
@@ -505,7 +583,7 @@ func (r *Reconciler) dispatcherDeploymentEnvVars(channel *kafkav1beta1.KafkaChan
 			Value: constants.DispatcherContainerName,
 		},
 		{
-			Name:  commonenv.KnativeLoggingConfigMapNameEnvVarKey,
+			Name:  commonconstants.KnativeLoggingConfigMapNameEnvVarKey,
 			Value: logging.ConfigMapName(),
 		},
 		{
@@ -538,24 +616,24 @@ func (r *Reconciler) dispatcherDeploymentEnvVars(channel *kafkav1beta1.KafkaChan
 		},
 	}
 
-	// If The Kafka Secret Env Var Is Specified Then Append Relevant Env Vars
-	if len(r.kafkaSecret) <= 0 {
+	// If The Kafka Secret Name Is Specified Then Append Relevant Env Vars
+	if len(r.config.Kafka.AuthSecretName) <= 0 {
 
-		// Received Invalid Kafka Secret - Cannot Proceed
-		return nil, fmt.Errorf("invalid kafkaSecret for topic '%s'", topicName)
+		// Received Invalid Kafka Secret Name - Cannot Proceed
+		return nil, fmt.Errorf("invalid authSecretName for topic '%s'", topicName)
 
 	} else {
 
 		// Append The Secret Name As Env Var
 		envVars = append(envVars, corev1.EnvVar{
 			Name:  commonenv.KafkaSecretNameEnvVarKey,
-			Value: r.kafkaSecret,
+			Value: r.config.Kafka.AuthSecretName,
 		})
 
 		// Append The Secret Namespace As Env Var
 		envVars = append(envVars, corev1.EnvVar{
 			Name:  commonenv.KafkaSecretNamespaceEnvVarKey,
-			Value: r.environment.SystemNamespace,
+			Value: r.config.Kafka.AuthSecretNamespace,
 		})
 
 	}
