@@ -22,26 +22,25 @@ import (
 	"strconv"
 	"time"
 
-	"knative.dev/eventing-kafka/pkg/apis/messaging/v1beta1"
-
-	"k8s.io/apimachinery/pkg/types"
-
 	"go.uber.org/zap"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/intstr"
+	"knative.dev/pkg/controller"
+	"knative.dev/pkg/logging"
+	"knative.dev/pkg/system"
+
+	"knative.dev/eventing-kafka/pkg/apis/messaging/v1beta1"
 	commonenv "knative.dev/eventing-kafka/pkg/channel/distributed/common/env"
 	"knative.dev/eventing-kafka/pkg/channel/distributed/common/health"
 	"knative.dev/eventing-kafka/pkg/channel/distributed/controller/constants"
 	"knative.dev/eventing-kafka/pkg/channel/distributed/controller/event"
 	"knative.dev/eventing-kafka/pkg/channel/distributed/controller/util"
 	commonconstants "knative.dev/eventing-kafka/pkg/common/constants"
-	"knative.dev/pkg/controller"
-	"knative.dev/pkg/logging"
-	"knative.dev/pkg/system"
 )
 
 // reconcileReceiver Reconciles The Receiver (Kafka Producer) For The Specified KafkaChannel
@@ -337,7 +336,7 @@ func (r *Reconciler) newReceiverDeployment(secret *corev1.Secret) *appsv1.Deploy
 	deploymentName := util.ReceiverDnsSafeName(r.config.Kafka.AuthSecretName)
 
 	// Replicas Int Value For De-Referencing
-	replicas := int32(r.config.Receiver.Replicas)
+	replicas := int32(r.config.Channel.Receiver.Replicas)
 
 	// Create The Receiver Container Environment Variables
 	channelEnvVars := r.receiverDeploymentEnvVars(secret)
@@ -347,18 +346,18 @@ func (r *Reconciler) newReceiverDeployment(secret *corev1.Secret) *appsv1.Deploy
 	// If we want "no limit" or "no request" then the entry must not be present in the map.
 	// Note: Since a "Quantity" type has no nil value, we use the Zero value to represent unlimited.
 	resourceLimits := make(map[corev1.ResourceName]resource.Quantity)
-	if !r.config.Receiver.MemoryLimit.IsZero() {
-		resourceLimits[corev1.ResourceMemory] = r.config.Receiver.MemoryLimit
+	if !r.config.Channel.Receiver.MemoryLimit.IsZero() {
+		resourceLimits[corev1.ResourceMemory] = r.config.Channel.Receiver.MemoryLimit
 	}
-	if !r.config.Receiver.CpuLimit.IsZero() {
-		resourceLimits[corev1.ResourceCPU] = r.config.Receiver.CpuLimit
+	if !r.config.Channel.Receiver.CpuLimit.IsZero() {
+		resourceLimits[corev1.ResourceCPU] = r.config.Channel.Receiver.CpuLimit
 	}
 	resourceRequests := make(map[corev1.ResourceName]resource.Quantity)
-	if !r.config.Receiver.MemoryRequest.IsZero() {
-		resourceRequests[corev1.ResourceMemory] = r.config.Receiver.MemoryRequest
+	if !r.config.Channel.Receiver.MemoryRequest.IsZero() {
+		resourceRequests[corev1.ResourceMemory] = r.config.Channel.Receiver.MemoryRequest
 	}
-	if !r.config.Receiver.CpuRequest.IsZero() {
-		resourceRequests[corev1.ResourceCPU] = r.config.Receiver.CpuRequest
+	if !r.config.Channel.Receiver.CpuRequest.IsZero() {
+		resourceRequests[corev1.ResourceCPU] = r.config.Channel.Receiver.CpuRequest
 	}
 
 	// If either the limits or requests are an entirely-empty map, this will be translated to a nil
@@ -382,7 +381,7 @@ func (r *Reconciler) newReceiverDeployment(secret *corev1.Secret) *appsv1.Deploy
 			Namespace: r.environment.SystemNamespace,
 			Labels: map[string]string{
 				constants.AppLabel:                  deploymentName, // Matches Service Selector Key/Value Below
-				constants.KafkaChannelReceiverLabel: "true",         // Allows for identification of Receivers
+				constants.KafkaChannelReceiverLabel: "true",         // Allows for identification of Receiver Deployments
 			},
 		},
 		Spec: appsv1.DeploymentSpec{
@@ -395,7 +394,8 @@ func (r *Reconciler) newReceiverDeployment(secret *corev1.Secret) *appsv1.Deploy
 			Template: corev1.PodTemplateSpec{
 				ObjectMeta: metav1.ObjectMeta{
 					Labels: map[string]string{
-						constants.AppLabel: deploymentName, // Matched By Deployment Selector Above
+						constants.AppLabel:                  deploymentName, // Matched By Deployment Selector Above
+						constants.KafkaChannelReceiverLabel: "true",         // Allows for identification of Receiver Pods
 					},
 					Annotations: map[string]string{
 						commonconstants.ConfigMapHashAnnotationKey: r.kafkaConfigMapHash,
@@ -494,7 +494,7 @@ func (r *Reconciler) receiverDeploymentEnvVars(secret *corev1.Secret) []corev1.E
 			Value: constants.ReceiverContainerName,
 		},
 		{
-			Name:  commonenv.KnativeLoggingConfigMapNameEnvVarKey,
+			Name:  commonconstants.KnativeLoggingConfigMapNameEnvVarKey,
 			Value: logging.ConfigMapName(),
 		},
 		{
